#!/usr/bin/perl -w -I/usr/lib/perl5/site_perl/

# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
# at first start up, we can be in any of the following states:
#
#    o just having been installed
#    o just having reboot from rebuilt kernel
#    o reboot requested by test (-f i-meant-to-reboot)
#    o unrequested reboot (error)
#
# potential HOST states: 
#
#   avail     - STP
#   maint     - Maintenance
#   * inst    - Installing OS
#   * comp    - Building Kernel
#   * runtest - Running Test
#   upload    - Uploading the results
#
#  * = Valid stp_client.pl entry points
#  
#  The file that means a reboot was requested on purpose is: i-meant-to-reboot
#  

use STP::Client::Transport;
use STP::XML::Config;
use STP::Util::Config;
use STP::XML::TestRequest;

use STP::Client::Tools;
use Sys::Hostname;
use Mail::Mailer;

use strict;

# Function declarations
# sub syslog;
# sub console_note;

syslog "STARTING STP CLIENT";

# programs we are using
my $plm_patch    = "/usr/local/bin/plm_build_tree.pl";
my $kernel_build = "stp_robokern";
my $lilo_install = "stp_fix-lilo.pl";
my $tar          = "/bin/tar";
my $testcmd      = "/usr/bin/test";
my $wget         = "/usr/bin/wget";
my $scp          = "/usr/bin/scp";
my $ping         = "/bin/ping";

# holds the state of the client hosts progress
my $host_state;

# Set this flag to have the script IGNORE ASP failures.
# regular behavior is to fail and kill the client.  sometimes
# we don't want that.
my $ignore_asp_failure = 0;

# Retry MAX_RETRY times with a timer of RETRY_TIMER seconds
my $MAX_RETRY   = 90;
my $RETRY_TIMER = 30;

# get our config
my $config = STP::Util::Config::getConfig();

# Get the current hostname
my $hostname = hostname();
if ( $hostname =~ /^(.+?)\./ ) {
    $hostname = $1;
}

# get our uri and proxy
my $uri   = $config->getElementValue( "service.uri" );
my $proxy = $config->getElementValue( "service.proxy" );

print "ASP: [$uri] $proxy\n";

# our current state
get_host_state();

# get our test info
my $testinfo = get_test_request();
unless ( $testinfo ) {
    syslog "PANIC: \$testinfo is null";
    console_note "PANIC: \$testinfo is null";
    exit 1;
}

# Extract useful information from $testinfo
my $test_name   = $testinfo->getElementValue( "test" );
my $test_uid    = $testinfo->getElementValue( "uid" );
my $patch_tag   = $testinfo->getElementValue( "patch_tag" );
my $patch_name  = $testinfo->getElementValue( "patch_name" ) || "";
my $distro_name = $testinfo->getElementValue( "distro_name" ) || "";
my $test_state  = $testinfo->getElementValue{ "state" } || "";

print "TEST INFO:\n" . $testinfo->toString() . "\n";

# *************************************************************
# *** CHECK TO MAKE SURE THE TEST AND HOST STATES ARE VALID ***
# *************************************************************

#
# *** These conditions indicate the stp_client.pl should not be running ***
# 

if ( $host_state eq "avail" ) {
    syslog "The host_state is 'avail' this indicates the host has not been "
      . "assigned a test yet.";
    syslog "This probably means I am not supposed to be running right now.";
    syslog "I can fail gracefully.";
    console_note "System avail - no current test assigned";

    exit( 1 );
}

if ( $host_state eq "maint" ) {
    syslog "The host_state is 'maint' this indicates this host is in "
      . "troubleshooting mode.";
    syslog "This probably means I am not supposed to be running right now.";
    syslog "I can fail gracefully to avoid getting in the way.";
    console_note "System detected maint state on reboot";

    exit( 1 );
}

unless ( ( defined $test_uid ) && ( $test_uid ) ) {
    syslog "I did not get a valid UID from the ASP server.";
    syslog "This probably means I am not supposed to be running right now.";
    syslog "I can fail gracefully.";
    console_note "System did not get valid test id from server. System idle";

    exit( 1 );
}

# 
# Check to see if test was canceled after we restarted the reimage
#
if ( $test_state eq "Cancelled" ) {
	syslog "The test was canceled";
	syslog "System being returned to available state";
	console_note "Test cancled: System available";
        set_host_state( "avail" );
	
	exit( 1 );
}
#
# *** This condition indicates the system did not reimage ***
#

if ( -f "request-details" ) {
    my $txt = `cat request-details`;
    $txt =~ /Request ID:\s*(\d+)/s;
    if ( $1 == $test_uid ) {
        syslog "OK: request-details found: ID [$test_uid] is verified";
    } else {
        syslog "PANIC: request-details found: ID [$1] != [$test_uid]";

        mail_user( "[STP $test_uid] Internal STP Failure - test requeued",
                   "The STP system had an internal failure and will adjust "
                   . "for it automatically.  Your test will be requeued and "
                   . "run again without any intervention required.\n\n"
                   . "The admins have been notified of this event."
                   . request_details() );

        mail_admin( "[STP $test_uid] Host $hostname failed to reimage",
                    request_details() );

        set_test_state( "Queued" );
        set_host_state( "avail" );
        console_note "System failed re-image - admin mail sent";

        exit( 1 );
    }
}

#
# *** These conditions indicate the test/host is in an invalid state ***
# 

if ( ( $host_state eq "runtest" ) && ( !-f 'i-meant-to-reboot' ) ) {
    syslog
      "The host_state is 'runtest' this indicates there has been a problem.";
    syslog "No indication of a reboot request was found.";
    syslog "It's possible the test failed and caused a reboot.";

    syslog "notifying the user the test failed to complete in time.";
    mail_user(
        "[STP $test_uid] Failed to complete $test_name on PLM $patch_tag",
        "The STP system was able to compile and boot the specified kernel.\n"
        . "A reboot happened while the system was running the test.  This is usually "
        . "a sign of instability in the kernel being tested.\n"
        . request_details() );

    syslog "Attempting to fail the test and re-queue the host.";

    upload_results();

    set_test_state( "Failed" );
    set_host_state( "avail" );
    console_note
      "Test failed due to timeout or other reboot condition - system available";

    exit( 1 );
}


if ( $host_state eq "upload" ) {
    syslog "The host_state is 'upload' this indicates there was a problem.";
    syslog "It's possible the system rebooted without getting reimaged.";
    console_note
      "The host_state is 'upload' this indicates there was a problem.";
    console_note "It's possible the system rebooted without getting reimaged.";

    syslog "Notifying the user and admin that there was an internal failure.";
    mail_user( "[STP $test_uid] Problem with $test_name on PLM $patch_tag",
"The STP system has had an internal problem when processing this test.\n"
        . "The request will be re-queued until it runs properly.\n"
        . request_details() );

    mail_admin( "[STP $test_uid] Possible failed reimage on $hostname",
           "The test will be re-queued and the host placed in available mode.\n"
           . request_details() );

    syslog "Attempting to re-queue the test and re-enable the host.";
    set_test_state( "Queued" );
    set_host_state( "avail" );

    exit( 1 );
}

if ( $host_state eq "comp" ) {
	syslog "The host_state is 'comp' this indicates the system rebooted while compiling the kernel";
	syslog "Notifying the user and admin that there was an internal failure.";
    mail_user( "[STP $test_uid] Problem with $test_name on PLM $patch_tag",
"The STP system has had an internal problem when processing this test.\n"
        . "The request will be re-queued until it runs properly.\n"
        . request_details() );

    mail_admin( "[STP $test_uid] Possible compile failure on $hostname",
           "The test will be re-queued and the host placed in available mode.\n"
           . request_details() );

    syslog "Attempting to re-queue the test and re-enable the host.";
    set_test_state( "Queued" );
    set_host_state( "avail" );

    exit( 1 );
}

# Test for system that fails reboot and does a second reboot on the old kernel

if ( $host_state eq "reboot" ) {
    # Test to be sure we are running the STP kernel
    my $lilo_out = `lilo -q | grep STP | grep '*'`;
    if( ! $lilo_out ) {
	syslog "The kernel failed to boot properly, and the system has restarted on a non-STP kernel";
        mail_user( "[STP $test_uid] Problem with $test_name on PLM $patch_tag",
          "The test kernel failed to boot properly.\n"
        . request_details() );
        mail_admin( "[STP $test_uid] failed to reboot '$host_state' on $hostname",
		"System has been placed in maint\n"
            . request_details() );
	set_host_state( "maint" );
	set_test_state( "Failed" );
	exit( 1 ); 
	
	}

}

unless ( ( $host_state eq "inst" )
	 || ( $host_state eq "reboot" )
         || ( $host_state eq "runtest" ) )
{
    syslog "The host_state is '$host_state' which is an UNKNOWN state.";

    syslog "Notifying the user and admin that there was an internal failure.";
    mail_user( "[STP $test_uid] Problem with $test_name on PLM $patch_tag",
"The STP system has had an internal problem when processing this test.\n"
        . "The request will be re-queued until it runs properly.\n"
        . request_details() );

    mail_admin( "[STP $test_uid] Unknown host_state '$host_state' on $hostname",
           "The test will be re-queued and the host placed in available mode.\n"
           . request_details() );

    syslog "Attempting to re-queue the test and re-enable the host.";
    set_test_state( "Queued" );
    set_host_state( "avail" );
    console_note
"The host_state is '$host_state' which is an UNKNOWN state. - test re-queued";

    exit( 1 );
}

# ****************************
# *** CLIENT GENERIC SETUP ***
# ****************************
#
# This code should be run regardless of which entry point we are heading for.
#

# 
# Basic who-am-I-and-what-am-I-doing info sent to syslog & stderr
# 
syslog "TEST REQUEST: ID: $test_uid, Test: $test_name, PLM: $patch_tag";
syslog
  "Valid state [$host_state] for host has been detected, we are ready to go.";

# echo it to the console so Ops and console tailers are happy
console_note "TEST REQUEST: ID: $test_uid, Test: $test_name, PLM: $patch_tag";
console_note
  "Valid state [$host_state] for host has been detected, we are ready to go.";

#
# Record the test details to disk if there are not already there
# 
if ( !-f 'request-details' ) {
    if ( open( F, ">request-details" ) ) {
        print F request_details();
        close F;
    } else {
        syslog "Unable to open request-details for writing.";
    }
}

# ****************************
# *** CLIENT ENTRY POINT 1 ***
# ****************************
#
# This entry point hist after the initial reimage
# 

if ( $host_state eq "inst" ) {
    system "mkdir logs";
    # These are routines to verify that the system is healthy
    # Processors
    # Memory
    # Disk
    my $sysstatus = system("/usr/bin/check_system.pl");
    my $sys_exit = $sysstatus / 256; # To get actual exit status
    if ( $sys_exit != 0 ) {
        console_note( "Failed Health Check" );
	set_test_state( "Queued" );
    	set_host_state( "maint" );
	exit( 1 );
	} 
    

    set_host_state( "comp" );    # HOST_STATE comp

    set_test_state( "Downloading Patch" );
    console_note( "Downloading Patch" );

    download_patch( $patch_tag );
    update_lilo();

    set_test_state( "Compiling Kernel" );
    console_note( "Compiling Kernel" );
    compile_kernel();

    $ignore_asp_failure = 1;
    set_host_state( "reboot" );
    set_test_state( "Rebooting After Kernel Install" );
    console_note( "Rebooting After Kernel Install" );

    reboot();
}

# ****************************
# *** CLIENT ENTRY POINT 2 ***
# ****************************
#
# This entry point hits after the system has booted the new kernel
# 

if ( $host_state eq "reboot" ) {
    
    set_host_state( "runtest" );    # HOST_STATE runtest

    set_test_state( "Downloading Test" );
    download_test( $test_name );

    set_test_state( "Unpacking Test" );
    unpack_test( $test_name );

    set_test_state( "Running Test" );
    console_note( "Running Test" );
    pre_save_environment();

    # no exit, we want to fallthrough to entry point 3
}

# ****************************
# *** CLIENT ENTRY POINT 3 ***
# ****************************
#
# This entry point only hits if either:
#   - test requested a reboot
#   - Entry point two just finished
#   

if ( $host_state eq "runtest" ) {
    syslog "Executing the test";

    # Clear out previous reboot requests
    system "rm -f i-meant-to-reboot";

    # Run the actual test
    my $result = run_test( $test_name );
    post_save_environment();

    # Upload the results to the ASP server
    set_host_state( "upload" );    # HOST_STATE upload
    set_test_state( "Uploading Results" );
    console_note( "Uploading results" );
    upload_results();

    # Tell the user we are done and mark the test as completed
    set_test_state( "Notifying User" );
    if ( $result == 0 ) {
        set_test_state( "Complete" );
        notify_user( $test_uid, $test_name );
    } else {
        set_test_state( "Failed" );
        notify_user_failed( $test_uid, $test_name );
    }

    # Free up this host for the next test
    set_host_state( "avail" );    # HOST_STATE avail
    console_note( "test completed - user notified - system available" );

    exit( 0 );
}

# ************************
# *** HELPER FUNCTIONS ***
# ************************

sub get_test_request {
    my $trxml = do_asp_action( "get_test_request_for_host", $hostname );
    my $tr = new STP::XML::TestRequest();
    $tr->parseXMLData( $trxml );

    return $tr;
}

sub download_patch {
    my ( $patch_tag ) = @_;

    system "$plm_patch linux $patch_tag";

    # We are done if the tree was created correctly
    # return if ( ( $? == 0 ) && ( -d "linux" ) );
    # Let's actually check for some errors
    # I'm repeating some code here, so it goes.
    #
    if ( $? != 0 ) {
        syslog "ERROR: Failure of patch script - patch_tag: $patch_tag";
        mail_user(
            "[STP $test_uid] Failure - Invalid Patch ID [Patch: $patch_tag]",
            "The STP patch script has failed. Please send details to "
            . "stp-devel\@lists.sourceforge.net. " );

        set_test_state( "Failed" );
        set_host_state( "avail" );

        exit( 1 );
    }
    if ( !-d "linux" ) {
        syslog "ERROR: Failure of kernel patch - patch_tag: $patch_tag";
        mail_user(
            "[STP $test_uid] Failure - Invalid Patch ID [Patch: $patch_tag]",
            "The STP was unable to download a kernel using the ID "
            . "you provided ($patch_tag).  This could be an internal "
            . "error, but it is more likely that the specified patch "
            . "ID does not exist.\n\n"
            . "Please verify that ID $patch_tag is valid, and email "
            . $config->getElementValue( "email" ) . " "
            . "if you have any questions.\n" );

        set_test_state( "Failed" );
        set_host_state( "avail" );

        exit( 1 );
    }

    # We know the directory exists 
    # Let's verify that things are cool
    chdir "./linux";

    # Check for patch.error file
    if ( -f "patch.error" ) {
        syslog "ERROR: Patch operation Failed - patch tag: $patch_tag";

        system( "cp patch.* ../logs" );

        mail_user(
"[STP $test_uid] Kernel Patch Error (patch: $patch_tag, $patch_name)",
"The STP was unable to patch the kernel. Below is the output from the build"
            . `cat ../patch.* ` );
        generate_index();

        upload_results();

        set_test_state( "Failed" );
        set_host_state( "avail" );

        exit( 1 );
    }

    chdir "..";
    return( 0 );

}

sub compile_kernel {
    my $res = 1;

    system "$kernel_build &> logs/kernel-build.txt";

    $res = 0 if ( $? );
    $res = 0 unless ( -f "linux/System.map" );
    $res = 0 unless ( -f "linux/arch/i386/boot/bzImage" );

    # We are done if there were no errors
    return 0 if ( $res );

    #my $patch_tag = $testinfo->getElementValue( "patch_tag" );

    syslog "ERROR: Unable to compile kernel: $patch_tag";

    mail_user(
        "[STP $test_uid] Kernel Compile Error (patch: $patch_tag, $patch_name)",
"The STP was unable to build the kernel using patch $patch_tag, $patch_name\n\n"
        . "Below is the output from the build:\n\n"
        . `tail -100 logs/kernel-build.txt`
        . "\n\nKernel Config File:\n"
        . `cat kconfig` );

    generate_index();

    upload_results();

    set_test_state( "Failed" );
    set_host_state( "avail" );

    exit( 1 );
}

sub reboot {
    syslog "A reboot has been requested.";

    system "/sbin/reboot";

    exit( 0 );
}

sub unpack_test {
    my $test = shift;

    syslog "Unpacking $test_name";

    system "$tar -xzvf $test.tar.gz";

    # We are done if there were no errors
    return if ( $? == 0 );

    syslog "PANIC: Failed unpacking $test_name";

    mail_user( "[STP $test_uid] Failed - Cannot unpack test $test_name",
               "The STP was unable to find test $test.  The Admin\n"
               . "has been notified as well.\n"
               . request_details() );

    mail_admin(
                "[STP $test_uid] Failed unpacking test $test_name",
                "STP Test $test failed to unpack.\n" . request_details()
    );

    set_test_state( "Failed" );
    set_host_state( "avail" );

    exit( 0 );
}

sub pre_save_environment {
    syslog "Saving the environment information (pre)";

    # save the environment data
    `rm -rf environment`;
    `mkdir environment`;
    if ( -f "/usr/sbin/lshw" ) {
	# We'll try both plain and fancy
    `/usr/sbin/lshw >environment/System_Information-before.txt 2> /dev/null`;
    `/usr/sbin/lshw -html >environment/System_Information-before.html 2> /dev/null`;
   } else {
    `si >environment/System_Information-before.txt 2> /dev/null`;
	}

    # check to see if the environment data is there correctly
    if ( !( -f "environment/System_Information-before.txt" ) ) {
        syslog "PANIC: Unable to save the environment info";

        mail_user( "[STP $test_uid] panic on internal error ",
                   "The STP was unable to check the system information.\n"
                   . "The Admin has been notified as well.\n"
                   . request_details() );

        mail_admin( "[STP $test_uid] panic on internal error",
                    "STP Test $test_uid failed to get system information.\n"
                    . request_details() );

        upload_results();

        set_test_state( "Failed" );
        set_host_state( "avail" );

        exit( 0 );
    }
}

sub run_test {
    my $test         = shift;
    my $ret          = 0;
    my $script_param = $testinfo->getElementValue( "script_param" );

    chdir $test;
    chmod 0755, "wrap.sh";
    system "stp_profile start";
    system "stp_oprofile start";
    system
". /usr/bin/stp_include.sh && ./wrap.sh $script_param &> ../logs/run-log.txt";
    $ret = -1 unless ( $? == 0 );
    system "stp_oprofile stop";
    system "stp_profile stop";

    chdir "..";

    return $ret;
}

sub post_save_environment {
    syslog "Saving the environment information (post)";

    `stp_sysinfo.sh`;

    # check to see if the environment data is there correctly
    if ( !( -f "environment/System_Information-after.txt" ) ) {
        syslog "PANIC: Missing environment info";
        syslog "You should never see this error.  Oops.";

        mail_user( "[STP $test_uid] Failure: missing environment info ",
                   "The STP was unable to check the system information.\n\n"
                   . "The Admin has been notified as well.\n"
                   . request_details() );

        mail_admin( "[STP $test_uid] Failure: missing environment info ",
                    "STP Test $test_uid failed to get system information.\n"
                    . request_details() );

        upload_results();

        set_test_state( "Failed" );
        set_host_state( "avail" );

        exit( 0 );
    }
}

#
# This function handles the generic uploading of results
#
# Future work could include reading a stp-upload file for a line-by-line of
# test-specific files that should be copied
#
sub upload_results {
    my @files;

    syslog "uploading results to the server";

    # Move the test results to the curent directory & archive them
    system "cp stp.log ./logs/stp.log.txt";
    system "mv $test_name/logs/* ./logs/";
    mkdir "./profile", 0777;
    system "mv $test_name/profile/* ./profile/";
    mkdir "./oprofile", 0777;
    system "mv $test_name/oprofile/* ./oprofile/";
    system "mv -f $test_name/results .";
    system "mv -f $test_name/*.html .";
    system "mv -f $test_name/result_email.txt .";

    # Build the list of files to upload
    push @files, "result_email.txt";    # Email content sent to user
    push @files, `ls *.html`;           # Test-generated html files
    push @files, "COPYING";             # Results license
    push @files, "logs";                # Output of wrap.sh
    push @files, "profile";             # Output of profiler
    push @files, "oprofile";            # Output of Oprofiler
    push @files, "environment";         # dir: Environment files
    push @files, "results";             # dir: actual test results

    # upload the files
    foreach my $file ( @files ) {
        chomp $file;

        # Auto-compress directories for upload 
        if ( -d $file ) {
            system "tar -zcf $file.tar.gz $file";
            $file = "$file.tar.gz";
        }

        # Skip this file if it's not present
        unless ( -f $file ) {
            syslog "file upload: $file failed, file missing";
            next;
        }

        # Do the actual upload
        syslog "file upload: $file";
        my $res = upload_file( $test_uid, $file );

        # Notify the user & admin if the file upload failed
        if ( $res != 1 ) {
            mail_user( "[STP $test_uid] File Upload Failure ($file)",
                "The STP was unable to upload results file \"$file\" "
                . "for test $test_uid.\n\n"
                . "This is a STP internal error, the Admin has been notified.\n"
                . request_details() );

            mail_admin( "[STP $test_uid] File Upload Failure ($file)",
                 "STP Test $test_uid failed to upload results file \"$file\".\n"
                 . request_details() );

        }
    }
}

#
# given the test_uid and a filename, upload the file to the ASP
# 
sub upload_file {
    my ( $test_uid, $filename ) = @_;

    my $encoded = `bzip2 -c $filename | uuencode -`;

    my $res =
      do_asp_action( "upload_test_results", $test_uid, $filename, $encoded );

    return $res;
}

sub notify_user {
    my ( $test_uid, $test ) = @_;

    mail_user( "[STP $test_uid] $test results ($patch_tag, $patch_name)",
               email_header() . `cat result_email.txt` );
}

sub notify_user_failed {
    my ( $test_uid, $test ) = @_;

    mail_user( "[STP $test_uid] $test run failed ($patch_tag, $patch_name)",
               email_header()
               . `cat result_email.txt` . "\n\n" . "=" x 60 . "\n\n"
               . `cat logs/run-log.txt` );
}

sub mail_admin {
    my ( $subject, @body ) = @_;

    syslog "Sending an email to the admin regarding [$subject]";

    send_mail( $config->getElementValue( "email" ), $subject, @body );
}

sub mail_user {
    my ( $subject, @body ) = @_;

    syslog "Sending an email to the user regarding [$subject]";

    send_mail( $testinfo->getElementValue( "email" ), $subject, @body );
}

#
# Do the actual work of sending the email to the admin or user
# 
sub send_mail {
    my ( $user, $subject, @body ) = @_;

    my $mailer = Mail::Mailer->new( "sendmail" );
    $mailer->open(
        {
            From    => $config->getElementValue( "email" ),
            To      => $user,
            Subject => $subject,
        }
    );

    print $mailer @body;
    $mailer->close();
}

#
# Generate and index.html when we fail to get as far as wrap.sh
#

sub generate_index {

    # This will be made in the execution directory, currently /root
    open( FH, ">index.html" );

    #  All these globals...
    print FH
"<HTML><HEAD><title>Scalable Test Platform Test Result</title><HEAD>\n<BODY>\n";
    print FH
"<H1>[STP $test_uid] Error in State $host_state (patch: $patch_tag, $patch_name)</H1>",
"<PRE>The STP was unable to build the kernel using patch $patch_tag, $patch_name\n\n"
      . "Below is the output from the build on $hostname:\n\n"
      . `tail -100 logs/kernel-build.txt`
      . "\n\nKernel Config File:\n"
      . `cat kconfig` . "\n";
    print FH "</PRE>\n";
    print FH "<a href=logs/>Log Outputs</a>\n</BODY>\n";
    close FH;
}

#
# This should pass the user-specified lilo option string to the lilo.conf file
#
sub update_lilo {
    my $lilo = $testinfo->getElementValue( "lilo" );

    return unless ( $lilo );

    open( FILE, ">lilo.opt" );
    print FILE $lilo;
    close( FILE );

    system "stp_fix-lilo.pl";
}

sub download_test {
    my ( $test ) = @_;

    my $tarline = $test . ".tar.gz";
    my $webpath = $config->getElementValue( "test.webpath" );

    $webpath .= $tarline;

    system $wget, $webpath;

    my $res = system $testcmd, "-x", $tarline;

    # if our wrap script does not exit
    if ( $res == 1 ) {
        mail_user( "[STP $test_uid] Failed to download test $test",
                   "The STP was unable to download test $test.\n\n"
                   . "The admin been notified as well.\n"
                   . request_details() );

        mail_admin( "[STP $test_uid] Failed to download test $test",
                    "STP Test $test failed to download.\n\n"
                    . "Webpath = $webpath\n"
                    . request_details() );

        set_test_state( "Failed" );
        set_host_state( "avail" );

        exit( 0 );
    }
}

#
# This function does the work of contacting the ASP server and handling the 
# retry timeouts.
#
# If we are unable to contact the server in the specified time, we email the 
# admin to let them know, but we also keep trying just in case the server 
# comes back before the stp_cron server times us out and fails the test.
# 
sub do_asp_action {
    my ( $command, @args ) = @_;
    my $res;

    my $retry_count = 0;

    # do the ASP action until either a proper result is returned,
    # or the retries are exhausted
    while ( $retry_count < $MAX_RETRY ) {
        syslog "do_asp_action() calling [$command] ("
          . scalar( @args )
          . " args)";
        $res = doASPAction( $uri, $proxy, $command, @args );

        return $res if ( defined $res );

        $retry_count++;
        sleep( $RETRY_TIMER );
    }

    return if ( $ignore_asp_failure );

    # by the time we get here, something is definitely wrong.
    # we can't set the state of the machine/test, so all we can
    # do is mail the admin and exit, hoping for the best

    my $proxy_host = "127.0.0.1";

    if ( $proxy =~ /http\:\/\/(.+?)\// ) {
        $proxy_host = $1;
    }

    my $txt;

    $txt = "STATE: $host_state\n\n" if $host_state;

    $txt .= `$ping -c 5 $hostname`;
    $txt .= `$ping -c 5 $proxy_host`;

    syslog "Warning the admins the do_asp_action call is failing.";

    mail_admin( "[STP $test_uid] $hostname cannot talk to ASP server",
                "STP retried $retry_count times, but was still unable "
                . "to contact the ASP server.\n"
                . "I will keep retrying until the server takes control.\n\n"
                . "\turi = [$uri]\n\tproxy = [$proxy]\n\n\n---\n$txt"
                . request_details() );

    $res = 0;
    while ( !$res ) {
        syslog "do_asp_action() calling [$command] (@args)";
        $res = doASPAction( $uri, $proxy, $command, @args );
    }
}

#
# Helper functions for set/get of the TEST state
# 
sub set_test_state {
    my $status = shift;

    syslog "Changing the test_status to '$status'";

    do_asp_action( "set_test_request_status", $test_uid, $status );
}

# 
# Helper functions for set/get of the HOST state
#
# We get the current state from the /tmp/stp-host-state file if it exists or
# the ASP server if not
#
sub get_host_state {
    my $old_value;

    # If the local file exists, the server will not have the correct value
    if ( ( !defined $host_state ) && ( -f "/tmp/stp-host-state" ) ) {
        $host_state = `cat /tmp/stp-host-state`;
        chomp $host_state;
        system "rm -f /tmp/stp-host-state";

        syslog "Current state: $host_state";

        # We only bail here if we got a valid entry from the file
        return if ( $host_state );
    }

    # We do not want to fail in the ASP code
    $old_value          = $ignore_asp_failure;
    $host_state         = do_asp_action( "get_host_state", $hostname );
    $ignore_asp_failure = $old_value;

    die "Unable to discover host state" unless ( defined $host_state );

    syslog "Current state: $host_state";
}

#
# We save the state localy in the /tmp directory and remotely on the ASP
# server.  In situations where the remote update fails, the stp_client.pl
# will restart with the state saved in the /tmp/stp-host-state file
#
sub set_host_state {
    $host_state = shift;
    my ( $old_value, $res );

    syslog "Setting host state to: $host_state";

    `echo $host_state > /tmp/stp-host-state` unless $host_state eq "avail";

    # We do not wait to fail out in the ASP 
    $old_value = $ignore_asp_failure;
    $res = do_asp_action( "set_host_state", $hostname, $host_state );
    $ignore_asp_failure = $old_value;

    # Delete the temporary disk file if the server got the message
    system "rm -f /tmp/stp-host-state" if ( defined $res );
}

#
# Create a string detailing the current request
#
# TODO replace globals with input parms
#
sub request_details {
    my $txt;
    my $lilo = $testinfo->getElementValue( "lilo" ) || "";
    my $script_param = $testinfo->getElementValue( "script_param" ) || "";

    # Do not edit the spacing on these without editing stp_include.sh
    $txt = "REQUEST DETAILS:\n";
    $txt .= "\tRequest ID:  $test_uid\n";
    $txt .= "\tTest Name:   $test_name\n";
    $txt .= "\tDistro Name: $distro_name\n";
    $txt .= "\tKernel ID:   $patch_tag\n";
    $txt .= "\tKernel Name: $patch_name\n";
    $txt .= "\tLILO opt:    $lilo\n";
    $txt .= "\nREQUEST INFO:\n";
    $txt .= "\tHost Name:   $hostname\n";
    $txt .= "\tHost State:  $host_state\n";
    $txt .= "\tScript Params: $script_param\n";
    $txt .= "\n";

    return $txt;
}

#
# Create the header information for result emails
#
sub email_header {
    my $txt;
    my $lilo = $testinfo->getElementValue( "lilo" ) || "";
    my $script_param = $testinfo->getElementValue( "script_param" ) || "";
    my $url = $config->getElementValue( "test.resulturl" )
      || "http://localhost";

    $txt .=
      "------------------------------------------------------------------\n\n";
    $txt .= "RESULTS for request ID [ $test_uid ]\n";
    $txt .=
"TEST: $test_name on $hostname running PLM ID $patch_tag [ $patch_name ]\n";
    $txt .= "LILO: append = '$lilo'\n";
    $txt .= "Script Params: $script_param\n";
    $txt .= "\n";
    $txt .= "Detailed Results: $url/$test_uid/\n";
    $txt .= "Environment Info: $url/$test_uid/environment/\n";
    $txt .= "Profile Details:  $url/$test_uid/profile/\n";
    $txt .=
      "------------------------------------------------------------------\n\n";

    return $txt;
}
