# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
package STP::ASP::TestRequest;

use STP::Validation::Suite;
use STP::DB::Database;
use STP::DB::TestRequest;
use STP::DB::Host;
use STP::DB::HostState;
use STP::XML::Config;
use STP::XML::Transport;
use STP::Util::Config;
use STP::Util::Log;
use STP::XML::TestRequest;

require Exporter;

@ISA = qw( Exporter );

@EXPORT = qw( get_test_request_for_host
  get_host_state
  set_host_state
  set_test_request_status
  upload_test_results
);

use strict;

my $log = new STP::Util::Log( ">>/tmp/stp.log" );

sub get_test_request_for_host {

    # shift away the ASP piece
    shift;

    my $validation =
      STP::Validation::Suite::validate( "get_test_request_for_host", @_ );

    # is everything kosher?
    if ( $validation == 0 ) {
        return 0;
    }

    my ( $hostname ) = @_;

    $log->log( "get_test_request_for_host( $hostname ) validated" );

    # figure out the parameters to pass to writeTestRequest
    my $config = STP::Util::Config::getConfig();

    my $tr = new STP::DB::TestRequest( $config );

    $tr->getTestRequestForHost( $hostname );
    my $xml = $tr->toXML();

    return $xml->toString();
}

sub set_host_state {

    # shift away the ASP piece
    shift;

    my $validation = STP::Validation::Suite::validate( "set_host_state", @_ );

    # is everything kosher?
    if ( $validation == 0 ) {
        return 0;
    }

    my ( $hostname, $state ) = @_;

    # figure out the parameters to pass to writeTestRequest
    my $config = STP::Util::Config::getConfig();

    # if we're updating the state to 0, we will delete the host mapping
    # hopefully to keep state sane.
    my $tr = new STP::DB::Host( $config );

    $tr->getHostByDescriptor( $hostname );

    if ( $tr->getNumRows() == 0 ) {
        return;
    }

    $tr->setHostState( $state );

    my $hostuid = $tr->getProperty( 0, "uid" );

    if ( $state eq "avail" ) {
        my $test = new STP::DB::TestRequest( $config );
        $test->removeTestRequestFromHost( $hostuid, "-1" );
    }
}

sub get_host_state {

    # shift away the ASP piece
    shift;

    $log->log( "get_host_state() called" );

    my $validation = STP::Validation::Suite::validate( "get_host_state", @_ );

    # is everything kosher?
    if ( $validation == 0 ) {
        return 0;
    }

    my ( $hostname ) = @_;

    $log->log( "get_host_state( $hostname ) call validated" );

    # figure out the parameters to pass to writeTestRequest
    my $config = STP::Util::Config::getConfig();

    my $tr     = new STP::DB::Host( $config );
    my $hstate = new STP::DB::HostState( $config );

    if ( $hostname =~ /^(.+?)\./ ) {
        $hostname = $1;
    }

    $tr->getHostByDescriptor( $hostname );

    my $ret = $tr->getHostState();

    $log->log( "get_host_state( $hostname ) returning $ret" );

    return $ret;
}

sub set_test_request_status {

    # shift away the ASP piece
    shift;

    my $validation =
      STP::Validation::Suite::validate( "set_test_request_status", @_ );

    # is everything kosher?
    if ( $validation == 0 ) {
        return 0;
    }

    my ( $test_request_uid, $status ) = @_;

    # figure out the parameters to pass to writeTestRequest
    my $config = STP::Util::Config::getConfig();

    my $tr = new STP::DB::TestRequest( $config );

    $tr->getTestRequestByUid( $test_request_uid );
    $tr->updateTestRequestStatus( $status );
}

sub upload_test_results {

    # shift away the ASP piece
    shift;

    my $validation =
      STP::Validation::Suite::validate( "upload_test_results", @_ );

    # is everything kosher?
    if ( $validation == 0 ) {
        return 0;
    }

    my ( $test_request_uid, $filename, $data ) = @_;

    # figure out where to write the results
    my $config = STP::Util::Config::getConfig();

    my $datapath = $config->getElementValue( "test.resultpath" );

    # sanity check
    die "upload_test_results() missing test.resultpath in config"
      unless $datapath;

    # figure out the directory to store things, create all of the directories
    my $uldir = "$datapath/$test_request_uid";

    $log->log( "upload_test_results() recieving $uldir/$filename" );

    # Set umask for results
    umask( 0002 );

    # make the directories
    if ( !-d $uldir ) {
        mkdir( $uldir, 0777 )
          || die "upload_test_results() cannot mkdir $uldir";
        mkdir( $uldir . "/results", 0777 );
    }

    # write the file
    my $wfile = "$uldir/$filename";
    open( F, "| uudecode | bunzip2 -c > $wfile" ) 
        or die "upload_test_results() cannot open target: $wfile\n[$!]";
    print F $data;
    close( F );

    if ( $filename =~ /tar\.gz/ ) {
        chdir $uldir;
        system "tar -zxf $filename";
        system "rm -f $filename";
        chdir "..";
    }

    return 1;
}

1;
