#!/usr/bin/perl -w

package OprofHandler;

# a tool to setup and run the oprofile toolset
#
use STP::Client::Tools;
use strict;
use Env;

sub new {
    my $class = shift;

    # All this goop should come from a config file.
    # preferrably a config file in cvs/cfengine
    my $self = {
        _wget_host      => "http://stp/data/stp-tools/",
        _oprof_tar      => "oprofile-0.7.1.tar.gz",
        _oprof_dir      => "oprofile-0.7.1",
        _oprof_build_v6 =>
          [ "./configure --with-kernel-support", "make", "make install" ],
        _oprof_build_v4 =>
          [ "./configure --with-linux=/root/linux", "make", "make install" ],

        _thome => $_[ 0 ],
        _opdir => "$_[0]" . "/oprofile"
    };
    bless $self, $class;

    # This also should be in a config file
    $self->{ _logfile }  = "$self->{_thome}/oprofile.txt";
    $self->{ _index }    = "$self->{_opdir}/index.html";
    $self->{ _op_cntrl } = "/usr/local/bin/opcontrol";
    $self->{ _op_rpt }   = "/usr/local/bin/opreport";
    $self->{ _op_note }  = "/usr/local/bin/opannotate";
    $self->{ _kern }     = "/boot/vmlinux";

    $self->_setup();
    $self->_init();
    return $self;

}

#
# Check for profile support and fail with a message if it's missing.
#
sub check_profile {
    my $self = shift;
    unless ( -d "/dev/oprofile" ) {
        console_note "This system is not configured for profile gathering.\n";
        console_note
          "To enable profile gathering make sure the kernel has profiling";
        console_note "enabled and/or the oprofile module is loaded.";

        $self->safe_exit();
    }
}

# Tells opcontrol where to find kernel
# Inits oprofile
sub _init {
    my $self = shift;
    if ( !-f $self->{ _opdir } ) {
        console_note "Making directory $self->{_opdir}";
        mkdir $self->{ _opdir };
        `ln -s $self->{_opdir} /oprofile`;
    }
    if ( -f "/usr/local/sbin/modprobe" ) {
        `/usr/local/sbin/modprobe -v oprofile`;
    } else {
        `modprobe -v oprofile`;
    }

    # we won't check status for this
    `$self->{_op_cntrl} --vmlinux=/root/linux/vmlinux`;
    `sleep 1`;
    `$self->{_op_cntrl} --init`;

    return 0;
}

# Recycle the daemon after report or snapshot
#
sub kick_daemon {
    my $self = shift;
    console_note "Recycling oprofile daemon";
    `$self->{_op_cntrl} --vmlinux=/root/linux/vmlinux`;
    `sleep 1`;
    `$self->{_op_cntrl} --start-daemon`;
    `sleep 1`;
    `$self->{_op_cntrl} --start`;
    `sleep 1`;
    `$self->{_op_cntrl} --reset `;
    return 0;

}

#
# Begin the profile gathering process, create the working dir
#
sub start {
    my $self = shift;
    $self->check_profile();

    console_note "Starting the oprofile gathering phase";
    $self->{ _index } = $self->{ _index };
    open( H, ">$self->{_index}" ) || die "Unable to open $self->{_index}$!";
    print H "<html>\n<head>\n<title>OProfile Data Report</title>\n</head>\n";
    print H "<body>\n";
    print H "<h1>OProfile Data Report</h1>\n";
    print H
      "OProfile data is available for the following timestamp events.<p>\n";
    print H "<table cellpadding=2 width=700>\n";
    close( H );

    $self->kick_daemon();
    `sleep 1`;
    $self->mark( "Framework Startup" );

    return 0;
}

sub mark {
    my ( $self, @marks ) = @_;
    $self->check_profile();
    my $txt = "Event: @marks @ " . `date`;
    chomp $txt;
    console_note $txt;

    my $tmp = join( "_", @marks );
    $tmp =~ s/\s\s+/ /g;
    $tmp =~ s/\W/_/g;

    console_note "Marking profile: $tmp\n";
    system "$self->{_op_cntrl} --dump";
    system "sleep 1";
    system "$self->{_op_cntrl} --save=$tmp";
    system "$self->{_op_cntrl} --reset";

# We could save the raw data!;
#system "cp -pR /var/lib/oprofile/samples/current $self->{_opdir}/$tmp-oprofile/";

    return 0;
}

sub add_to_html {
	my ( $self, $filename, $label, @event ) = @_;
    my $txt = "Event: @event @ " . `date`;
    chomp $txt;

    my $tmp = join( "_", @event );
    $tmp =~ s/\s\s+/ /g;
    $tmp =~ s/\W/_/g;
    $txt =~ /Event:\s+(.+)\s+@\s+(.+)$/;
    open( H, ">>$self->{_index}" ) || die "Unable to open $self->{_index} $!";
    print H "<tr>";
    print H "<td>$1</td>";
    print H "<td>$2</td>";
    print H "<td><a href='$filename'>$label</a></td>";
    # print H
# "<td><a href='$tmp-oprofile-annotated'>OProfile Annotated Source</a></td>";
    # print H
# "<td><a href='$tmp-oprofile-assembly.txt'>OProfile Annotated Assembly</a></td>";

    #print H "<td><a href='$tmp-oprofile'>Raw-data</a></td>";
    print H "</tr>\n";
    close( H );
    return 0;
}

sub report_kern {

    my $self = shift;
    $self->check_profile();
    my $tmp = join( "_", @_ );
    $tmp =~ s/\s\s+/ /g;
    $tmp =~ s/\W/_/g;
    console_note "Reporting oprofile event: $tmp";
    `$self->{_op_cntrl} --dump`;
`$self->{_op_rpt} -l $self->{_kern} -o $self->{_opdir}/$tmp-kernel.oprofile.txt session:$tmp`;
    $self->add_to_html( "$tmp-kernel.oprofile.txt", "Kernel Oprofile Data", @_ );
    return 0;
}

sub report_all {

    my $self = shift;
    $self->check_profile();
    my $tmp = join( "_", @_ );
    $tmp =~ s/\s\s+/ /g;
    $tmp =~ s/\W/_/g;
    console_note "Reporting oprofile event: $tmp";
    `$self->{_op_cntrl} --dump`;
    `$self->{_op_rpt} -o $self->{_opdir}/$tmp-all.oprofile.txt session:$tmp`;
    $self->add_to_html( "$tmp-all.oprofile.txt", "System Oprofile Data", @_ );
    return 0;
}

sub anote_src {
    my $self = shift;
    $self->check_profile();
    my $tmp = join( "_", @_ );
    $tmp =~ s/\s\s+/ /g;
    $tmp =~ s/\W/_/g;
    console_note "Annotatin oprofile event: $tmp";
    `$self->{_op_cntrl} --stop`;
`$self->{_op_note} --source --output-dir=$self->{_opdir}/$tmp-oprofile-annotated`;
    $self->kick_daemon();
    $self->add_to_html( "$tmp-oprofile-annotated", "Annotated Source Directory", @_ );
    return 0;
}

sub anote_assembly {
    my $self = shift;
    $self->check_profile();
    my $tmp = join( "_", @_ );
    $tmp =~ s/\s\s+/ /g;
    $tmp =~ s/\W/_/g;
    console_note "Annotatin oprofile event: $tmp";
    `$self->{_op_cntrl} --stop`;
`$self->{_op_note} --source --assembly session:$tmp > $self->{_opdir}/$tmp-oprofile-assembly.txt 2>&1`;
    $self->kick_daemon();
    $self->add_to_html( "$tmp-oprofile-assembly.txt", "Annotated Assembly", @_ );
    return 0;
}

sub stop {
    my $self = shift;
    $self->check_profile();

    $self->mark( "Framework Close" );
    console_note "Ending the profile gathering phase";

    open( H, ">>$self->{_index}" ) || die "Unable to open $self->{_index}: $!";
    print H "\n</table><p>\n";
    print H "The log file is available <a href='profile.log.txt'>here</a>.\n";
    print H "</body>\n</html>";
    close( H );
    `$self->{_op_cntrl} --stop`;

    $self->safe_exit();
}

sub safe_exit {
    my $self = shift;
    my $val  = shift || 0;

    system "mv $self->{_logifile} $self->{_opdir}/.";

    return $val;
}

sub _setup {

    my $self = shift;

    # This is the only way we know to unwind the array
    my $build_ref;
    if ( -f "/usr/local/bin/opcontrol" ) { return 0; }
    my $cur_dir = `pwd`;
    chdir "/root";
    my $res = get_bits( $self->{ _oprof_tar }, $self->{ _wget_host } );
    if ( $res == -1 ) {
        console_note
          "Failed to get bits $self->{_oprof_tar} from $self->{_wget_host}\n";
        return 1;
    }
    my $PATCHLEV =
      `grep "PATCHLEVEL =" /root/linux/Makefile | sed 's/PATCHLEVEL = //'`;
    if ( $PATCHLEV == 6 ) {
        $build_ref = $self->{ _oprof_build_v6 };
        $res = install_tar(
                            $self->{ _oprof_dir },
                            $self->{ _oprof_tar },
                            @$build_ref
        );
    } elsif ( $PATCHLEV == 4 ) {
        $build_ref = $self->{ _oprof_build_v4 };
        $res = install_tar(
                            $self->{ _oprof_dir },
                            $self->{ _oprof_tar },
                            @$build_ref
        );
    } else {
        console_note "Cannot determine kernel version\n";
        return 1;
    }
    if ( $res == -1 ) {
        console_note "Failed to install oprofile\n";
        `cp /tmp/$self->{_oprof_dir}.log .`;
        return 1;
    }

    # Fix binary location
    `ln -s /root/linux/vmlinux /boot/vmlinux`;
    chdir $cur_dir;
    return 0;
}
1;
