# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::DB::TestRequest;

@ISA = qw( STP::DB::Database );

use strict;
use warnings;
use STP::XML::TestRequest;
use STP::DB::Database;
use STP::DB::Host;
use STP::DB::Test;
use STP::DB::User;
use STP::Util::Log;
use POSIX qw(strftime);

BEGIN { }

my $log = new STP::Util::Log( ">>/tmp/stp.log" );

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    # TODO - what the heck does db_lock do? 
    # We set it, but never query it...

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " test_request  ";

    $self->{ fields } =
      "tr.uid, tr.descriptor, tr.created_by, tr.created_date, "
      . "tr.status, tr.test_uid, tr.script_param, tr.patch_tag, "
      . "tr.auto_publish, tr.lilo, tr.test_priority, tr.host_type_uid, "
      . "tr.host_uid, tr.completion_date, tr.distro_tag_uid";

    return $self;
}

# create the table (only used at setup time)
sub createTable {
    my ( $self ) = @_;

    my $query = qq(
		    CREATE TABLE $self->{ tableName }
		    ( uid INT, descriptor TEXT, created_by INT,
		      created_date DATETIME, status VARCHAR(255),
		      test_uid INT, script_param VARCHAR(255),
		      patch_tag VARCHAR(255),
		      auto_publish INT, lilo VARCHAR(255),
		      test_priority INT, host_type_uid INT,
		      host_uid INT, completion_date DATETIME
		    )
		   );

    $self->_update( $query );

    $query = qq(
		CREATE TABLE test_request_to_host
		( test_request_uid INT, host_uid INT )
		);
    $self->_update( $query );
}

sub getTestRequestByUid {
    my ( $self, $uid ) = @_;

    unless ( $uid ) {
        $log->log( "getTestRequestByUid: MISSING UID" );
        return 0;
    }
    my $query =
      "SELECT $self->{ fields } "
      . "FROM $self->{ tableName } tr "
      . "WHERE uid = $uid";

    $log->log( "getTestRequestByUid: $query" );
    $self->{ resultSet } = $self->_select( $query );
}

sub getOldestQueuedTestByPriority {
    my ( $self, $priority, $hosttype ) = @_;

    my $query =
      "SELECT $self->{ fields } FROM $self->{ tableName } tr "
      . "WHERE tr.status = 'QUEUED' AND tr.test_priority = $priority "
      . "AND tr.host_type_uid = $hosttype ORDER BY tr.created_date";

    #$log->log( "getOldestQueuedTestByPriority: $query" );
    $self->{ resultSet } = $self->_select( $query );
}

sub getCancelledTests {
    my ( $self, $priority, $hosttype ) = @_;

    my $query = qq( 
		   SELECT $self->{ fields }
		   FROM $self->{ tableName } tr
		   WHERE tr.status = 'CANCEL' 
		   ORDER BY tr.created_date
		  );

    $self->{ resultSet } = $self->_select( $query );

    my $num = $self->getNumRows();
    $log->log( "getCancelledTests: found $num tests" );
    return $num;
}

sub writeTestRequest {
    my (
         $self,              $uid,             $descriptor,
         $created_by,        $created_date,    $last_updated_by,
         $last_updated_date, $status,          $test_uid,
         $script_param,      $patch_tag,       $auto_publish,
         $lilo,              $completion_date, $test_priority,
         $host_type_uid
      )
      = @_;

    my $query;

    # do we insert or update?
    if ( $uid == 0 ) {

        # we're inserting, so get a new uid
        $query = qq(
		    INSERT INTO $self->{ tableName }
		    ( descriptor, created_by, created_date, status,
		      test_uid, script_param, patch_tag,
		      auto_publish, lilo, test_priority, host_type_uid )
		    VALUES
		    ( '$descriptor', $created_by, '$created_date',
		      '$status', $test_uid, '$script_param', 
		      '$patch_tag', $auto_publish, '$lilo', $test_priority,
		      $host_type_uid
		    )
		   );

        $log->log( "query = \n$query\n" );

    }

    # update
    else {
        $query =
          "UPDATE $self->{ tableName } "
          . "SET descriptor = '$descriptor', created_by = $created_by, "
          . "created_date = '$created_date', status = '$status', "
          . "test_uid = $test_uid, script_param = '$script_param', "
          . "patch_tag = '$patch_tag', auto_publish = $auto_publish, "
          . "lilo = '$lilo', test_priority = $test_priority, "
          . "host_type_uid = $host_type_uid WHERE uid = $uid";
    }

    $self->_update( $query );

    return $uid;
}

sub updateTestRequestStatus {
    my ( $self, $status ) = @_;

    my $uid = $self->getProperty( 0, "uid" );

    my $query = qq(UPDATE test_request SET status = '$status' WHERE uid = $uid);

    $self->_update( $query );

    # if the status is "Complete" or "Failed", then update the competion date
    if ( $status eq "Complete" || $status eq "Failed" ) {
        my $completion_date = strftime( "%Y-%m-%d %H:%M:%S", localtime );

        $query =
qq(UPDATE test_request SET completion_date = '$completion_date' WHERE uid = $uid);
        $self->_update( $query );
    }
}

sub removeTestRequestFromHost {
    my ( $self, $host_uid, $test_uid ) = @_;

    my $query = qq(DELETE FROM test_request_to_host WHERE host_uid = $host_uid OR test_request_uid = $test_uid);

    $self->_update( $query );
}

sub assignTestRequestToHost {
    my ( $self, $test_uid, $host_uid ) = @_;

    # if we're assigning a test to a host, we need to make sure we removed the
    # old one
    $self->removeTestRequestFromHost( $host_uid, $test_uid );

    sleep 1;   # Allow db lock to be removed

    my $query =
      "INSERT INTO test_request_to_host ( test_request_uid, host_uid ) "
      . "VALUES ( $test_uid, $host_uid )";

    my $rv=0;
    my $counter=0;
    $rv=$self->_update( $query );
    while ($rv < 1 && $counter < 4) {
        sleep 1;                # Give database lock time to be removed
        $rv=$self->_update( $query );
        $counter+=1;
    }
    if ($rv < 1){
        $log->log( "getTestRequestForHost:  Failed to insert into test_request_to_host." );
        return -1;
    }

    # update the test request to include the host uid
    $query =
      qq(UPDATE test_request SET host_uid = $host_uid WHERE uid = $test_uid);

    $self->_update( $query );
    return 0;
}

# given a hostname, return the current test assigned to it
sub getTestRequestForHost {
    my ( $self, $hostname ) = @_;

    unless ( $hostname ) {
        $log->log( "getTestRequestForHost: MISSING HOSTNAME" );
        exit 1;
    }

    if ( $hostname =~ /^(.+?)\./ ) {
        $hostname = $1;
    }

    my $query =
      "SELECT trh.test_request_uid AS test_request_uid, trh.host_uid "
      . "FROM test_request_to_host trh, host h "
      . "WHERE h.descriptor = '$hostname' AND h.uid = trh.host_uid ";

    $self->_select( $query );

    my $truid = $self->getProperty( 0, "test_request_uid" );

    $log->log( "getTestRequestForHost: $truid" );

    # now, load ourselves as the test request, then return the uid
    if ( $truid ) { $self->getTestRequestByUid( $truid ) }

    return $truid;
}

# given a host uid, return the current test assigned to it
sub getTestRequestForHostUid {
    my ( $self, $hostuid ) = @_;

    my $query =
qq(SELECT trh.test_request_uid AS test_request_uid, trh.host_uid FROM test_request_to_host trh, host h WHERE h.uid = $hostuid AND h.uid = trh.host_uid );

    $self->_select( $query );

    my $truid = $self->getProperty( 0, "test_request_uid" );

    $log->log( "getTestRequestForHost: hostuid: $hostuid -> test: $truid" );

    # now, load ourselves as the test request, then return the uid
    $self->getTestRequestByUid( $truid ) if ( $truid );

    return $truid;
}

# given a uid, return the tests created by the user
sub getTestRequestsForUid {
    my ( $self, $uid ) = @_;

    my $query =
qq(SELECT $self->{ fields } FROM $self->{ tableName } tr WHERE tr.created_by = $uid ORDER BY uid desc);

    $self->_select( $query );

    return $self->arrayOfResults();
}

# given a test uid and host uid, determine what command should be used to 
# install the distro on the remote machine
sub getInstallCommand {
    my ( $self, $test_uid, $host_uid ) = @_;

    my $sql;

    $sql =
      "SELECT install_command.command "
      . "FROM install_command, install_req, install_req_to_distro_tag, "
      . "host_type_to_install_req, host_type, distro_tag, test_request, host "
      . "WHERE test_request.uid = $test_uid && "
      . "host.uid = $host_uid && "
      . "host_type.uid = host.host_type_uid && "
      . "distro_tag.uid = test_request.distro_tag_uid && "
      . "host_type_to_install_req.host_type_uid = host_type.uid && "
      . "install_req.uid = host_type_to_install_req.install_req_uid && "
      . "install_req_to_distro_tag.install_req_uid = install_req.uid && "
      . "install_req_to_distro_tag.distro_tag_uid = distro_tag.uid && "
      . "install_command.uid = install_req_to_distro_tag.install_command_uid";

    $self->_select( $sql );

    return $self->getProperty( 0, "command" );
}

sub arrayOfResults() {
    my ( $self ) = @_;

    my $count = $self->getNumRows();

    # if there are no results, return an undef
    if ( $count == 0 ) {
        return undef;
    }

    my @data = ();

    for ( my $i = 0; $i < $count; $i++ ) {
        my $uid = $self->getProperty( $i, "uid" );

        # pass our config along
        my $t = new STP::DB::TestRequest( $self->{ config } );

        $t->getTestRequestByUid( $uid );
        $data[ $i ] = $t;
    }

    return @data;
}

sub toXML {
    my ( $self ) = @_;
    my $xml = new STP::XML::TestRequest();

    $xml->setElementValue( "uid",  $self->getProperty( 0, "uid" ) );
    $xml->setElementValue( "lilo", $self->getProperty( 0, "lilo" ) );
    $xml->setElementValue( "script_param",
                           $self->getProperty( 0, "script_param" ) );
    $xml->setElementValue( "distro_tag", $self->getProperty( 0, "distro_tag_uid" ) );
    $xml->setDistroName( $self->{ config } );
    $xml->setElementValue( "auto_publish",
                           $self->getProperty( 0, "auto_publish" ) );
    $xml->setElementValue( "patch_tag", $self->getProperty( 0, "patch_tag" ) );
    $xml->setPatchName( $self->{ config } );

    $log->log( "toXML: status = " . $self->getProperty( 0, "status" ) );
    $xml->setElementValue( "status", $self->getProperty( 0, "status" ) );

    $xml->setElementValue( "completion_date",
                           $self->getProperty( 0, "completion_date" ) );

    # find out the test name, and write it
    my $test_uid = $self->getProperty( 0, "test_uid" );

    my $test = new STP::DB::Test( $self->{ config } );
    $test->getTestByUid( $test_uid );

    $xml->setElementValue( "test", $test->getProperty( 0, "descriptor" ) );

    # set the email address to send reports to
    my $ah = $self->getProperty( 0, "host_uid" );
    $log->log( "assigned_host = $ah" );

    my $user_uid = $self->getProperty( 0, "created_by" );

    my $user = new STP::DB::User( $self->{ config } );
    $user->getUserByUid( $user_uid );

    $xml->setElementValue( "email", $user->getProperty( 0, "real_email" ) );

    # set the host that the test is running on
    my $host = new STP::DB::Host( $self->{ config } );

    $host->getHostByUid( $ah );
    if ( $host->getNumRows() != 0 ) {
        $xml->setElementValue( "host", $host->getProperty( 0, "descriptor" ) );
    }

    return $xml;
}

END { }

1;
