# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::Test::Harness;

use Time::localtime;
use Time::HiRes qw(gettimeofday tv_interval);

# instanciate a new Harness
sub new {
    my $self = {};
    bless $self;
    shift;

    # set the number of tests and the number of completed tests both to zero
    $self->{ numTests }       = 0;
    $self->{ completedTests } = 0;

    return $self;
}

# run a test, record the results
sub test {
    my $self = shift;

    my ( $testname, $expectedResult, @args ) = @_;

    # which test number are we on?
    my $testnum = $self->{ numTests };

    # increment the number of tests
    $self->{ numTests }++;

    # add the test to the results
    $self->{ "test" . $testnum } = $testname;

    # Set our timestamp
    $self->{ "start" . $testnum } = [ gettimeofday ];

    # actually execute the test
    my $returnValue = $testname->( @args );

    # Calculate the elapsed time
    $self->{ "time" . $testnum } = tv_interval( $self->{ "start" . $testnum } );

    # did we pass the test or not?
    my $passfail = 0;

    if ( $returnValue eq $expectedResult ) {
        $passfail = 1;
    }

    $self->{ "expected" . $testname } = $expectedResults;
    $self->{ "actual" . $testname }   = $returnValue;

    # set our results
    $self->{ "results" . $testnum } = $passfail;

    # show that we've completed another test
    $self->{ completedTests }++;

}

# prints a report if requested, and returns the percentage of successful
# tests
sub report {
    my ( $self ) = shift;

    my ( $verbose ) = @_;

    if ( !defined( $verbose ) ) {
        $verbose = 2;
    }

    # number of passed/failed tests
    my $pass = 0;
    my $fail = 0;

    if ( $verbose >= 1 ) {
        print "Test Results\n";
        print "------------\n";

        print "\n";
        print $self->{ completedTests } . " tests completed.\n";
    }

    # if we're really verbose, iterate through the results
    if ( $verbose >= 2 ) {
        print "\n";
    }

    for ( my $i = 0; $i < $self->{ numTests }; $i++ ) {
        if ( $verbose >= 2 ) {
            print $self->{ "test" . $i };
            print " " x ( 60 - length( $self->{ "test" . $i } ) );

        }

        if ( $self->{ "results" . $i } ) {
            if ( $verbose >= 2 ) {
                print "[ PASS ]  ";
            }

            if ( $verbose >= 3 ) {
                print $self->{ "time" . $i };
            }

            print "\n";

            $pass++;
        } else {
            if ( $verbose >= 2 ) {
                print "[ FAIL ]\n";
            }
            $fail++;
        }
    }

    my $percentage = int( 10000 * $pass / ( $pass + $fail ) ) / 100;

    if ( $verbose >= 1 ) {
        print "\n";
        print "Total PASS: " . $pass . "\n";
        print "Total FAIL: " . $fail . "\n\n";
        print "Percentage Passed: " . $percentage . "\n\n";
    }

    return $percentage;
}

1;
