#!/usr/bin/perl -w -I/usr/lib/perl5/site_perl/

=head1 NAME


Simulated STP Client

=head1 SYNOPSIS

A method of testing the STP scheduler without having to wait for test
machine to reload.

=head1 DESCRIPTION

This module will provide a machine object to be used by a load
simulator. The machine object will use the STP database for state,
and control state by changing tables in the STP database. 

The machine will include:

a host_uid
a host_type_uid
a test_uid
a test_cost - which will decrement each time around the wheel

=cut

package machine;

use STP::Client::Transport;
use STP::XML::Config;
use STP::Util::Config;
use STP::XML::TestRequest;

use strict;
use warnings;

=head3 FUNCTION new

Creates a new machine. Machine holds most of the state from test_reqeust
Each test is represented by a 'test cost' represented by a counter. 
This is so the test will stay in state 'test running' for a few cycles
of the engine

The new machine is intialized with a host_uid, host_type_uid and a 
host_name

=cut

# potential HOST states:
#
#   avail     - STP
#   maint     - Maintenance
#   * inst    - Installing OS
#   * comp    - Building Kernel
#   * runtest - Running Test
#   upload    - Uploading the results

sub new {
    my ( $pkg, $host_uid, $host_type_uid, $host_name ) = @_;

    # Internal Control
    my $stp_state     = 8;
    my $machine_state = "new";

    # We initalize differenty, due to re-imaging

    # Test data to be filled in per test
    my $test_name   = 0;
    my $test_cost   = 0;
    my $patch_tag   = 0;
    my $test_row    = 0;
    my $patch_name  = "";
    my $distro_name = "";

    my $config = STP::Util::Config::getConfig();
    my $uri    = $config->getElementValue( "service.uri" );
    my $proxy  = $config->getElementValue( "service.proxy" );

    my $stp_machine = bless {
                              host_uid    => $host_uid,
                              host_type   => $host_type_uid,
                              test_name   => $test_name,
                              cost        => $test_cost,
                              stp_state   => $stp_state,
                              host_state  => $machine_state,
                              hostname    => $host_name,
                              config      => $config,
                              uri         => $uri,
                              proxy       => $proxy,
                              patch_tag   => $patch_tag,
                              patch_name  => $patch_name,
                              distro_name => $distro_name,
                              test_row    => $test_row
    }, $pkg;
    return $stp_machine;
}

# This is a simple set of accessor functions from the adv perl book

sub host_name {
    my $obj = shift;
    @_ ? $obj->{ 'hostname' } = shift: $obj->{ 'hostname' };
}

sub host_id {
    my $obj = shift;
    @_ ? $obj->{ 'host_uid' } = shift: $obj->{ 'host_uid' };
}

sub host_type {
    my $obj = shift;
    @_ ? $obj->{ 'host_type' } = shift: $obj->{ 'host_type' };
}

sub uri { my $obj = shift; @_ ? $obj->{ 'uri' } = shift: $obj->{ 'uri' }; }

sub proxy {
    my $obj = shift;
    @_ ? $obj->{ 'proxy' } = shift: $obj->{ 'test_name' };
}

sub test_row {
    my $obj = shift;
    @_ ? $obj->{ 'test_row' } = shift: $obj->{ 'test_row' };
}

sub test_id {
    my $obj = shift;
    @_ ? $obj->{ 'test_name' } = shift: $obj->{ 'test_name' };
}

sub cost { my $obj = shift; @_ ? $obj->{ 'cost' } = shift: $obj->{ 'cost' }; }

sub stp_state {
    my $obj = shift;
    @_ ? $obj->{ 'stp_state' } = shift: $obj->{ 'stp_state' };
}

sub host_state {
    my $obj = shift;
    @_ ? $obj->{ 'host_state' } = shift: $obj->{ 'host_state' };
}

sub patch_tag {
    my $obj = shift;
    @_ ? $obj->{ 'patch_tag' } = shift: $obj->{ 'patch_tag' };
}

sub patch_name {
    my $obj = shift;
    @_ ? $obj->{ 'patch_name' } = shift: $obj->{ 'patch_name' };
}

sub distro_name {
    my $obj = shift;
    @_ ? $obj->{ 'distro_name' } = shift: $obj->{ 'distro_name' };
}

sub runit {
    
    my $obj = shift;
    my $tmp = $obj->cost();
	print "Running current cost $tmp\n";
    $tmp--;
    $obj->cost( $tmp );
}

sub panic {
    my ( $obj, $msg ) = @_;
    printf "ERROR %s\n", $msg;
    exit 1;
}

sub set_test_state {

    my ( $obj, $msg ) = @_;
    my $res = doASPAction( $obj->{ 'uri' },
                       $obj->{ 'proxy' },
                       "set_test_request_status", $obj->{ 'test_row' }, $msg, );
    print "Set_test_state $obj->{ 'test_row'} $msg\n";
    return $res if ( defined $res );
    $obj->panic( "set_test_state" );
}

sub set_host_state {
    my ( $obj, $msg ) = @_;
    my $res = doASPAction( $obj->{ 'uri' },
                           $obj->{ 'proxy' },
                           "set_host_state", $obj->{ 'hostname' }, $msg, );
    $obj->host_state( $msg );
    print "Set_host_state $obj->{ 'hostname' } $msg\n";
    return $res if ( defined $res );
    $obj->panic( "set_host_state" );
}

sub get_host_state {
    my  $obj = shift;
    my $res = doASPAction( $obj->{ 'uri' },
                           $obj->{ 'proxy' },
                           "get_host_state", $obj->{ 'hostname' }  );
    return $res if ( defined $res );
    $obj->panic( "get_host_state" );
}

sub add_cost {
    my $obj = shift;
    $obj->{ 'cost' } = 0;
  SWITCH: {
        if ( $obj->{ 'test_name' } eq 'hackbench' ) {
            $obj->{ 'cost' } = 1;
            last SWITCH;
        }
        if ( $obj->{ 'test_name' } eq 'tiobench' ) {
            $obj->{ 'cost' } = 5;
            last SWITCH;
        }
        if ( $obj->{ 'test_name' } eq 'reaim' ) {
            $obj->{ 'cost' } = 4;
            last SWITCH;
        }
        if ( $obj->{ 'test_name' } eq 'iozone' ) {
            $obj->{ 'cost' } = 6;
            last SWITCH;
        }
        if ( $obj->{ 'test_name' } eq 'dbt2-1tier' ) {
            $obj->{ 'cost' } = 7;
            last SWITCH;
        }
        if ( $obj->{ 'test_name' } eq 'bonnie' ) {
            $obj->{ 'cost' } = 2;
            last SWITCH;
        }
        if ( $obj->{ 'test_name' } eq 'lmbench3_long' ) {
            $obj->{ 'cost' } = 8;
            last SWITCH;
        }
    }
    if ( $obj->{ 'cost' } eq 0 ) { $obj->{ 'cost' } = 3; }

    return $obj;
}

=head3 FUNCTION test_setup

The transition is a bit goofy - in the real STP, the machine transitions
from state 'avail' to state 'inst' due to an external power cycle/reimage.
For this dummy, we'll try a separate test_setup funtion. The client driver
will call this function to restart a machine with a new test. 

=cut

sub test_setup {
    my $obj   = shift;
    my $trxml = doASPAction( $obj->{ 'uri' },
                             $obj->{ 'proxy' },
                             "get_test_request_for_host", $obj->{ 'hostname' },
    );
    my $tr = new STP::XML::TestRequest();
    $tr->parseXMLData( $trxml );
	
    unless ( $tr->getElementValue( "test" ) ) {
        $obj->set_host_state( "avail" );
        return 1;
    }
    $obj->test_id( $tr->getElementValue( "test" ) );
    $obj->add_cost();
    $obj->patch_tag( $tr->getElementValue( "patch_tag" ) );
    $obj->patch_name( $tr->getElementValue( "patch_name" ) );
    $obj->distro_name( $tr->getElementValue( "distro_name" ) );
    $obj->test_row( $tr->getElementValue( "uid" ) );
    my $oldstate = $obj->get_host_state();
    if ( defined $oldstate ) {
	$obj->host_state( $oldstate );
	} else {
    	$obj->set_host_state( "inst" );
	}
    print
"Test setup complete - test $obj->{ 'test_row' } is $obj->{ 'test_name' }\n";
    return 0;
}

=head3 FUNCTION run

This is where everything happens. We determine initial state, 
and go from there. 

State transition actions involve telling the database a few things.

=cut

sub run {

    my $obj = shift;

    # today i outline the basic state transitions.
    my $h_state = $obj->host_state();
    printf "%s is in %s\n",$obj->host_name(), $obj->host_state();

  SWITCH: {
        if ( $h_state eq 'inst' ) {
		print"IN INST\n";
            $obj->set_host_state( "comp" );
            $obj->set_test_state( "Downloading Patch" );
            $obj->set_test_state( "Compiling Kernel" );
            $obj->set_test_state( "Rebooting After Kernel Install" );
            last SWITCH;
        }
        if ( $h_state eq  'comp' ) {
            $obj->set_host_state( "runtest" );
            $obj->set_test_state( "Downloading Test" );
            $obj->set_test_state( "Unpacking Test" );
            $obj->set_test_state( "Running Test" );
            last SWITCH;
        }
        if ( $h_state eq  'runtest' ) {
            if ( $obj->cost() > 0 ) {
                $obj->runit();
                last SWITCH;
            } else {
                $obj->set_host_state("upload" );
                $obj->set_test_state("Uploading results" );
                $obj->set_test_state("Notifying User" );
                $obj->set_test_state("Complete" );
                $obj->set_host_state("avail" );
                last SWITCH;
            }
        }
        if ( $h_state eq 'avail' ) {
            # printf" %s available \n", $obj->host_name();
            $obj->test_setup();
            last SWITCH;
        }
        if ( $h_state eq  'maint' ) {
            printf" ERROR %s in maint \n", $obj->host_name();
	    last SWITCH;
        }
        if ( $h_state eq 'new' ) {
            $obj->test_setup();
            last SWITCH;
        }

    }
    return $obj->host_state();
}

1;
